/***********************************************************************
 * Linotte                                                             *
 * Version release date : July 30, 2008                                *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.frame.atelier;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.linotte.frame.Atelier;
import org.linotte.frame.latoile.LaToile;
import org.linotte.moteur.entites.Acteur;
import org.linotte.moteur.entites.Casier;
import org.linotte.moteur.entites.Prototype;
import org.linotte.moteur.entites.Role;
import org.linotte.moteur.exception.ErreurException;
import org.linotte.moteur.outils.Chaine;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.api.Librairie;

@SuppressWarnings("serial")
public class Inspecteur extends JDialog {

	private static DefaultMutableTreeNode rootNode;
	private static DefaultTreeModel treeModel;
	private static JTree tree;
	private Librairie lib;
	private static JButton buttonVider;
	private static JButton buttonEffacer;
	private static JButton button;
	private static JLabel buttonDebogage;

	public Inspecteur(Librairie lib, Atelier a) {
		super(a, ModalityType.MODELESS);
		setTitle("Inspecteur");
		setSize(new java.awt.Dimension(400, 300));
		setMinimumSize(new java.awt.Dimension(400, 300));
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.lib = lib;
		rootNode = new DefaultMutableTreeNode("Inspecteur");
		treeModel = new DefaultTreeModel(rootNode);
		tree = new JTree(treeModel);
		tree.setSize(new java.awt.Dimension(400, 300));
		tree.setMinimumSize(new java.awt.Dimension(400, 300));
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setShowsRootHandles(false);
		tree.setRootVisible(false);
		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setLeafIcon(Ressources.getImageIcon("list-remove.png"));
		tree.setCellRenderer(renderer);
		tree.expandRow(0);
		setIconImage(Ressources.getImageIcon("utilities-system-monitor.png").getImage());
		JScrollPane scrollPane = new JScrollPane(tree);
		JMenuBar bar = new JMenuBar();
		button = new JButton(Ressources.getImageIcon("view-refresh.png"));
		button.setText("Rafraîchir");
		button.setMnemonic(KeyEvent.VK_R);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e0) {
				refresh(null);
			}
		});
		bar.add(button);

		buttonVider = new JButton(Ressources.getImageIcon("mail-mark-junk.png"));
		buttonVider.setText("Vider la mémoire");
		buttonVider.setMnemonic(KeyEvent.VK_V);
		buttonVider.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e0) {
				vider();
			}
		});
		// bar.add(buttonVider);

		buttonEffacer = new JButton(Ressources.getImageIcon("edit-clear.png"));
		buttonEffacer.setText("Effacer la toile");
		buttonEffacer.setMnemonic(KeyEvent.VK_E);
		buttonEffacer.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e0) {
				effacer();
			}
		});
		// bar.add(buttonEffacer);

		buttonDebogage =  new JLabel() ;
		bar.add(buttonDebogage);
		
		add(scrollPane);
		setJMenuBar(bar);

		Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (int) ((dimension.getWidth() - getWidth()) - 50);
		int y = (int) ((dimension.getHeight() - getHeight()) / 50);
		setLocation(x, y);

	}

	protected void effacer() {
		for (Object otoile : lib.getToiles()) {
			((LaToile) otoile).getPanelLaToile().effacer();
		}
	}

	public static void lock(boolean v) {
		buttonVider.setEnabled(v);
		buttonEffacer.setEnabled(v);
		button.setEnabled(true);
		buttonDebogage.setText("");
	}

	private void vider() {
		effacer();
		lib.vider();
		refresh(null);
	}

	@SuppressWarnings("unchecked")
	public void refresh(Map<Chaine, Acteur> pacteurs) {
		rootNode.removeAllChildren();
		Map<String, Acteur> acteurs = lib.getActeurs();
		synchronized (acteurs) {
			Set<String> list = acteurs.keySet();
			List<String> list2 = new ArrayList<String>();
			for (String nom : list) {
				list2.add(nom);
			}
			if (pacteurs != null) {
				button.setEnabled(false);
				// Variables du paragraphe :
				addObject(null, "--------------- Acteurs de la fonction :");
				for (Chaine clef : pacteurs.keySet()) {
					try {
						addObject(null, toString(clef.toString(), pacteurs.get(clef)));
					} catch (ErreurException e) {
					}
				}
				addObject(null, "--------------- Acteurs de la mémoire globale :");
			} else
				button.setEnabled(true);

			Collections.sort(list2);
			for (String nom : list2) {
				Acteur acteur = acteurs.get(nom);
				try {
					addObject(null, toString(null, acteur));
				} catch (ErreurException e) {
					addObject(null, getRole(acteur) + "(" + acteur.getNom() + ")=" + e.getMessage());
					e.printStackTrace();
				}
			}
		}
		treeModel.reload();
	}

	private String toString(String nom, Acteur acteur) throws ErreurException {
		if (acteur instanceof Prototype) {
			StringBuilder builder = new StringBuilder();
			builder.append(nom != null ? nom : acteur.getNom() + " " + "(" + getRole(acteur) + ")=");
			Prototype espece = (Prototype) acteur;
			Map acteurs = espece.retourAttributsMap();
			Iterator i = espece.retourAttributs().iterator();
			builder.append("{ ");
			for (; i.hasNext();) {
				builder.append(toString(null, (Acteur) acteurs.get(i.next()))).append(i.hasNext() ? " , " : "");
			}
			builder.append(" }");
			return builder.toString();
		} else if (acteur instanceof Casier) {
			StringBuilder builder = new StringBuilder();
			builder.append(nom != null ? nom : acteur.getNom() + " " + "(" + getRole(acteur) + ")=");
			Casier casier = (Casier) acteur;
			List acteurs = (List) casier.getValeur();
			builder.append("{ ");
			for (Iterator i = acteurs.iterator(); i.hasNext();) {
				builder.append(toString(null, (Acteur) i.next())).append(i.hasNext() ? " , " : "");
			}
			builder.append(" }");
			return builder.toString();
		} else {
			return (nom != null ? nom : (acteur.getNom() == null ? "?" : acteur.getNom())) + " " + "(" + getRole(acteur) + ")=" + acteur.getValeur();
		}
	}

	private String getRole(Acteur acteur) {
		if (acteur.getRole() == Role.NOMBRE)
			return "nombre";
		else if (acteur.getRole() == Role.TEXTE)
			return "texte";
		else if (acteur.getRole() == Role.CASIER) {
			return "casier de " + getRole(((Casier) acteur).roleContenant());
		} else if (acteur.getRole() == Role.ESPECE) {
			return ((Prototype) acteur).getType();
		} else
			return "?";
	}

	private String getRole(Role role) {
		if (role == Role.NOMBRE)
			return "nombre";
		else if (role == Role.TEXTE)
			return "texte";
		else if (role == Role.CASIER) {
			return "casier";
		} else if (role == Role.ESPECE) {
			return "espèce";
		} else
			return "?";
	}

	public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, Object child) {
		DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);

		if (parent == null) {
			parent = rootNode;
		}
		treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

		return childNode;
	}

	@Override
	public void setVisible(boolean b) {
		if (!isVisible())
			super.setVisible(b);
	}

	public JLabel getLabelDebogage() {
		return buttonDebogage;
	}

}
