package org.linotte.frame.cahier;

/***********************************************************************
 * Langage Linotte                                                     *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 3 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.dnd.DropTarget;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import javax.swing.AbstractAction;
import javax.swing.JEditorPane;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;
import javax.swing.text.html.HTML;
import javax.swing.undo.UndoManager;

import org.linotte.frame.Atelier;
import org.linotte.frame.Atelier.CopyL;
import org.linotte.frame.Atelier.CutL;
import org.linotte.frame.Atelier.PasteL;
import org.linotte.frame.Atelier.RedoAction;
import org.linotte.frame.Atelier.UndoAction;
import org.linotte.frame.cahier.sommaire.JPanelSommaire;
import org.linotte.frame.coloration.AtelierDocument;
import org.linotte.frame.coloration.ProcessStyle;
import org.linotte.frame.coloration.StyleBuffer;
import org.linotte.frame.coloration.StyleLinotte;
import org.linotte.frame.favoris.Favoris;
import org.linotte.frame.gui.JTextPaneText;
import org.linotte.frame.gui.PopupListener;
import org.linotte.moteur.outils.FichierOutils;
import org.linotte.moteur.outils.Preference;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Linotte;
import org.linotte.moteur.xml.analyse.Grammaire;

/**
 * 
 * Classe pour l'affichage du cahier
 * 
 * http://www.daniweb.com/software-development/java/threads/331500/how-can-i-add
 * -a-clickable-url-in-a-jtextpane
 * 
 * @author ronan
 * 
 */
@SuppressWarnings("serial")
public class Cahier extends JPanel implements KeyListener, MouseListener {

	static final Cursor handCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
	static final Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR);
	static final Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);

	public enum EtatCachier {
		SAUVEGARDE, MODIFIE
	}

	private static final int UPDATE_TIME = 2000;

	private JPopupMenu popupCahier;

	public ProcessStyle processStyle = null;

	protected JtextAreaCompletion jEditorPaneCachier = null;

	protected UndoManager undoManager = null;

	private boolean editeurUpdate = false;

	private long lastupdate = 0;

	private Atelier atelier;

	private File fichier;

	private JPanelSommaire jPanelSommaire = null;

	// Gestion de l'affichage des couleurs dans l'Atelier

	private StyleBuffer styleBuffer = null;

	private Onglet onglet;

	private EtatCachier etatCahier = EtatCachier.MODIFIE;

	// Favoris :
	public Set<Favoris> favoris = new HashSet<Favoris>();

	// Vue package :
	public JScrollPane scrollPan;

	private boolean gestionFavoris = true;

	// Tests unitaires :
	public boolean tests;

	// Historique de navigation dans le fichier

	private Stack<Integer> pileHistorique = new Stack<Integer>();

	protected class DocumentListenerImpl implements DocumentListener {
		public void insertUpdate(DocumentEvent e) {
			if (e.getType() != DocumentEvent.EventType.CHANGE) {
				setEtatCahier(EtatCachier.MODIFIE);
			}
			setEditeurUpdate(true);
			if (isGestionFavoris())
				for (Favoris fav : favoris) {
					fav.insert(e.getLength(), e.getOffset());
				}
			//processApplicationStyle.appliquer(30, 0);
		}

		public void removeUpdate(DocumentEvent e) {
			if (e.getType() != DocumentEvent.EventType.CHANGE) {
				setEtatCahier(EtatCachier.MODIFIE);
				if (isGestionFavoris())
					for (Favoris fav : favoris) {
						fav.remove(e.getLength(), e.getOffset());
					}
			}
			setEditeurUpdate(true);
			//processApplicationStyle.appliquer(30, 0);
		}

		public void changedUpdate(DocumentEvent e) {
			//processApplicationStyle.appliquer(30, 0);
		}
	}

	protected class UndoableEditEcouteur implements UndoableEditListener {
		public void undoableEditHappened(UndoableEditEvent e) {
			getUndoManager().addEdit(e.getEdit());
			atelier.undoAction.updateUndoState();
			atelier.redoAction.updateRedoState();
		}
	}

	class Save extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			atelier.jButtonRanger.doClick();
		}
	}

	class Close extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			onglet.button.doClick();
		}
	}

	class Next extends AbstractAction {
		public Next() {
			super();
		}

		public Next(String string) {
			super(string);
		}

		public void actionPerformed(ActionEvent e) {
			atelier.getCahierOnglet().next();
		}
	}

	class Previous extends AbstractAction {
		public Previous() {
			super();
		}

		public Previous(String string) {
			super(string);
		}

		public void actionPerformed(ActionEvent e) {
			atelier.getCahierOnglet().previous();
		}
	}

	class Max extends AbstractAction {
		public Max() {
			super();
		}

		public Max(String string) {
			super(string);
		}

		public void actionPerformed(ActionEvent e) {

			if (Atelier.explorateur.isVisible()) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						Atelier.explorateur.setVisible(false);
						atelier.tabbedPaneNavigateur.setVisible(false);
					}
				});
			} else {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						Atelier.explorateur.setVisible(true);
						atelier.tabbedPaneNavigateur.setVisible(true);
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								atelier.splitPaneProjet.setDividerLocation(atelier.taille_split_projet);
							}
						});
					}
				});
			}
		}
	}

	class Retour extends AbstractAction {

		public Retour() {
			super();
		}

		public Retour(String string) {
			super(string);
		}

		public void actionPerformed(ActionEvent e) {
			if (!pileHistorique.isEmpty()) {
				jEditorPaneCachier.setCaretPosition(pileHistorique.pop());
			}
		}
	}

	public Cahier() {
	}

	public void init(UndoAction undoAction, RedoAction redoAction, CutL couper, CopyL copier, PasteL coller, Font font, Atelier ppere, Linotte linotte) {

		atelier = ppere;

		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(getJEditorPaneCahier(), BorderLayout.CENTER);
		panel.setBackground(Color.WHITE);

		scrollPan = new JScrollPane();
		scrollPan.getVerticalScrollBar().setUnitIncrement(10);
		scrollPan.getHorizontalScrollBar().setUnitIncrement(10);

		LineRenderer lineRenderer = new LineRenderer(this);
		setLayout(new BorderLayout());
		add(lineRenderer, BorderLayout.WEST);
		add(scrollPan, BorderLayout.CENTER);
		scrollPan.getViewport().setView(panel);
		jEditorPaneCachier.setLineRenderer(lineRenderer);

		// Popup pour activer la couleur :
		popupCahier = new JPopupMenu();

		popupCahier.add(undoAction).setIcon(Ressources.getImageIcon("edit-undo.png"));
		popupCahier.add(redoAction).setIcon(Ressources.getImageIcon("edit-redo.png"));

		JMenuItem cut = new JMenuItem("Couper"), copy = new JMenuItem("Copier"), paste = new JMenuItem("Coller");

		cut.addActionListener(couper);
		copy.addActionListener(copier);
		paste.addActionListener(coller);

		copy.setIcon(Ressources.getImageIcon("edit-copy.png"));
		paste.setIcon(Ressources.getImageIcon("edit-paste.png"));
		cut.setIcon(Ressources.getImageIcon("edit-cut.png"));

		popupCahier.add(new Retour("Retour         Alt+Gauche"));
		popupCahier.add(new Max("Agrandir       Crt+M"));
		popupCahier.add(new Next("Cahier svt     Crt+Page haut"));
		popupCahier.add(new Next("Cahier prcd    Crt+Page bas"));
		popupCahier.add(copy);
		popupCahier.add(paste);
		popupCahier.add(cut);

		MouseListener popupListenerCahier = new PopupListener(popupCahier);

		undoManager = new CahierUndoManager();
		getDocumentCahier().addUndoableEditListener(new UndoableEditEcouteur());

		jEditorPaneCachier.getActionMap().put("Undo", undoAction);
		jEditorPaneCachier.getActionMap().put("Redo", redoAction);
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control Z"), "Undo");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control Y"), "Redo");

		// createActionTable(jEditorPaneCachier);

		jEditorPaneCachier.getActionMap().put("Copier", copier);
		jEditorPaneCachier.getActionMap().put("Coller", coller);
		jEditorPaneCachier.getActionMap().put("Couper", couper);
		jEditorPaneCachier.getActionMap().put("Save", new Save());
		jEditorPaneCachier.getActionMap().put("Close", new Close());
		jEditorPaneCachier.getActionMap().put("Next", new Next());
		jEditorPaneCachier.getActionMap().put("Previous", new Previous());
		jEditorPaneCachier.getActionMap().put("Max", new Max());
		jEditorPaneCachier.getActionMap().put("Retour", new Retour());
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control X"), "Couper");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control V"), "Coller");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control C"), "Copier");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control S"), "Save");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control W"), "Close");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control M"), "Max");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control PAGE_UP"), "Next");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("control PAGE_DOWN"), "Previous");
		jEditorPaneCachier.getInputMap().put(KeyStroke.getKeyStroke("alt LEFT"), "Retour");

		setEditorPaneTabs(jEditorPaneCachier, 2);
		jEditorPaneCachier.setDragEnabled(true);
		jEditorPaneCachier.setOpaque(false);
		jEditorPaneCachier.setForeground(Color.BLUE);
		jEditorPaneCachier.getStyledDocument().addDocumentListener(new DocumentListenerImpl());

		jEditorPaneCachier.addStyle("normal", StyleLinotte.styleRacine);

		jEditorPaneCachier.setFont(font);

		jEditorPaneCachier.addMouseListener(popupListenerCahier);
		jEditorPaneCachier.addKeyListener(this);
		jEditorPaneCachier.addMouseListener(this);

		new DropTarget(jEditorPaneCachier, new DragAndDropBoardListener(atelier));

		jPanelSommaire = new JPanelSommaire(this);

		styleBuffer = new StyleBuffer();

		processStyle = new ProcessStyle(linotte, this, styleBuffer);
		processStyle.start();

		LinkController handler = new LinkController();
		jEditorPaneCachier.addMouseMotionListener(handler);
		jEditorPaneCachier.addMouseListener(handler);

	}

	/**
	 * This method initializes jEditorPaneCachier
	 * 
	 * @return JEditorPane
	 */
	private JEditorPane getJEditorPaneCahier() {
		// TODO remplacer l'appel Ã grammaire
		List<String> t = new ArrayList<String>();
		t.addAll(Grammaire.getMots());
		jEditorPaneCachier = new JtextAreaCompletion(this, new AtelierDocument(this), t, true);
		return jEditorPaneCachier;
	}

	public String getSelectedText() {
		return jEditorPaneCachier.getSelectedText();
	}

	public void replaceSelection(String param) {
		jEditorPaneCachier.replaceSelection(param);
	}

	public UndoManager getUndoManager() {
		return undoManager;
	}

	public void setCaretPosition(int pos) {
		jEditorPaneCachier.setCaretPosition(pos);
	}

	public void setCaretPositionAndScroll(int pos) {
		jEditorPaneCachier.setCaretPosition(pos);
		Rectangle r;
		try {
			r = jEditorPaneCachier.modelToView(pos);
			if (r != null) {
				Rectangle vis = jEditorPaneCachier.getVisibleRect(); // to get
																		// the
																		// actual
																		// height
				r.height = vis.height;
				jEditorPaneCachier.scrollRectToVisible(r);
			}
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} // scroll to position 0, i.e. top

	}

	public void focus() {
		jEditorPaneCachier.grabFocus();
	}

	public int getCaretPosition() {
		return jEditorPaneCachier.getCaretPosition();
	}

	public Document getDocumentCahier() {
		if (jEditorPaneCachier != null)
			return jEditorPaneCachier.getDocument();
		else
			return null;
	}

	public JtextAreaCompletion getEditorPaneCahier() {
		return jEditorPaneCachier;
	}

	public void setSelectionCachier(int debut, int fin) throws BadLocationException {
		jEditorPaneCachier.setCaretPosition(debut);
		jEditorPaneCachier.setSelectionStart(debut);
		jEditorPaneCachier.setSelectionEnd(fin);
		jEditorPaneCachier.grabFocus();
	}

	public void setPositionCahier(int debut) throws BadLocationException {
		try {
			jEditorPaneCachier.setCaretPosition(debut);
			// http://forum.java.sun.com/thread.jspa?threadID=5300883&tstart=90
			Rectangle mRectangle2 = jEditorPaneCachier.modelToView(debut);
			Rectangle mRectangle3 = new Rectangle(mRectangle2.x, mRectangle2.y, mRectangle2.width, jEditorPaneCachier.getVisibleRect().height);
			jEditorPaneCachier.scrollRectToVisible(mRectangle3);
		} catch (Exception e) {
			// goFinTableau();
		}
		jEditorPaneCachier.grabFocus();
	}

	public void effacerCahier() {
		Document doc = getDocumentCahier();
		synchronized (doc) {
			try {
				getDocumentCahier().remove(0, getDocumentCahier().getLength());
			} catch (BadLocationException e2) {
				e2.printStackTrace();
			}
		}
		undoManager.discardAllEdits();
		jEditorPaneCachier.setCaretPosition(0);
	}

	public void setEditeurUpdate(boolean editeurUpdate) {
		if (editeurUpdate) {
			lastupdate = System.currentTimeMillis();
			processStyle.setStopLecture();
		}
		this.editeurUpdate = editeurUpdate;
	}

	public String retourneTexteCahier() {
		try {
			Document in = getDocumentCahier();
			return in.getText(0, in.getLength());
		} catch (BadLocationException e) {
			return "";
		}
	}

	public boolean isEditeurUpdate() {
		return editeurUpdate
		// Soulage le CPU...
				&& (System.currentTimeMillis() - lastupdate) > UPDATE_TIME;
	}

	public void ecrireInsertCachier(String texte) {
		try {
			this.getDocumentCahier().insertString(getDocumentCahier().getLength(), texte, null);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}

	public void ecrireInsertAuCurseurCachier(String texte) {
		try {
			this.getDocumentCahier().insertString(getCaretPosition(), texte, null);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}

	// http://forum.java.sun.com/thread.jspa?forumID=57&threadID=585006
	public void setEditorPaneTabs(JTextPaneText textPane, int charactersPerTab) {
		FontMetrics fm = textPane.getFontMetrics(textPane.getFont());
		int charWidth = fm.charWidth('w');
		int tabWidth = charWidth * charactersPerTab;
		ShowParLabelView.decallage = tabWidth;

		TabStop[] tabs = new TabStop[20];

		for (int j = 0; j < tabs.length; j++) {
			int tab = j + 1;
			tabs[j] = new TabStop(tab * tabWidth);
		}

		TabSet tabSet = new TabSet(tabs);
		SimpleAttributeSet attributes = new SimpleAttributeSet();
		StyleConstants.setTabSet(attributes, tabSet);
		int length = textPane.getDocument().getLength();
		textPane.getStyledDocument().setParagraphAttributes(0, length, attributes, false);
	}

	public File getFichier() {
		return fichier;
	}

	public void sauvegarderFichier(File fichier) throws IOException {
		this.fichier = fichier;
		FichierOutils.ecrire(fichier, retourneTexteCahier());
		setEtatCahier(EtatCachier.SAUVEGARDE);
		changeAtelierEtat();
	}

	public void chargerFichier(File fichier) throws IOException {
		effacerCahier();
		this.fichier = fichier;
		ecrireInsertCachier(FichierOutils.lire(fichier));
		setCaretPosition(0);
		getJSommaire().vider();
		setEtatCahier(EtatCachier.SAUVEGARDE);
		changeAtelierEtat();
	}

	public void renommerFichier(File fichier) throws IOException {
		this.fichier = fichier;
		onglet.setTitre();
	}

	public void forceStyle() {
		processStyle.force();
	}

	public void changeAtelierEtat() {
		boolean modifie = etatCahier == EtatCachier.MODIFIE;
		if (getFichier() == null) {
			Preference.getIntance().setProperty(Preference.P_FICHIER, "");
			atelier.setTitle("(nouveau livre) - " + Atelier.NAME);
		} else {
			Preference.getIntance().setProperty(Preference.P_FICHIER, getFichier().getAbsolutePath());
			atelier.setTitle(getFichier().getAbsolutePath() + " - " + Atelier.NAME);
		}
		atelier.jButtonRanger.setEnabled(modifie);
		atelier.getJButtonTester().setVisible(tests);
		atelier.undoAction.updateUndoState();
		atelier.redoAction.updateRedoState();
	}

	public JPanelSommaire getJSommaire() {
		return jPanelSommaire;
	}

	public StyleBuffer getStyleBuffer() {
		return styleBuffer;
	}

	public void setOnglet(Onglet buttonTabComponent) {
		onglet = buttonTabComponent;
	}

	public Onglet getOnglet() {
		return onglet;
	}

	public EtatCachier getEtatCahier() {
		return etatCahier;
	}

	public void setEtatCahier(EtatCachier etat) {
		// if (etat != etatCahier) {
		atelier.jButtonRanger.setEnabled(etat == EtatCachier.MODIFIE);
		etatCahier = etat;
		onglet.setTitre();
		// }
	}

	public Atelier getAtelier() {
		return atelier;
	}

	public void setCouleur(Color couleur, boolean force) {
		if (onglet != null)
			onglet.setCouleur(couleur, force);
	}

	public void setGestionFavoris(boolean gestionFavoris) {
		this.gestionFavoris = gestionFavoris;
	}

	public boolean isGestionFavoris() {
		return gestionFavoris;
	}

	public class LinkController extends MouseAdapter implements MouseMotionListener {

		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2 && !e.isConsumed()) {

				e.consume();
				JTextPane editor = (JTextPane) e.getSource();
				Document doc = editor.getDocument();
				Point pt = new Point(e.getX(), e.getY());
				int pos = editor.viewToModel(pt);

				if (pos >= 0) {
					if (doc instanceof DefaultStyledDocument) {
						DefaultStyledDocument hdoc = (DefaultStyledDocument) doc;
						Element el = hdoc.getCharacterElement(pos);
						AttributeSet a = el.getAttributes();
						String href = (String) a.getAttribute(HTML.Attribute.HREF);

						if (href != null) {
							try {
								pileHistorique.push(jEditorPaneCachier.getCaretPosition());
								getJSommaire().forceParagraphe(href);
							} catch (Exception ev) {
								System.err.println(ev.getMessage());
							}
						}
					}// if (doc instanceof DefaultStyledDocument)
				}// if (pos >= 0)
					// handle double click.
			}

		}// public void mouseClicked(MouseEvent e)

		public void mouseMoved(MouseEvent ev) {

			JTextPane editor = (JTextPane) ev.getSource();
			Point pt = new Point(ev.getX(), ev.getY());
			int pos = editor.viewToModel(pt);

			if (pos >= 0) {
				Document doc = editor.getDocument();

				if (doc instanceof DefaultStyledDocument) {
					DefaultStyledDocument hdoc = (DefaultStyledDocument) doc;
					Element e = hdoc.getCharacterElement(pos);
					AttributeSet a = e.getAttributes();
					String href = (String) a.getAttribute(HTML.Attribute.HREF);

					if (href != null) {
						if (getCursor() != handCursor) {
							if (getJSommaire().isParagraphe(href))
								editor.setCursor(handCursor);
							else
								editor.setCursor(defaultCursor);
						}
					} else {
						editor.setCursor(defaultCursor);
					}
				}// (doc instanceof DefaultStyledDocument)
			}// pos >=0
			else {
				setToolTipText(null);
			}
		}// mouseMoved
	}// LinkController

	@Override
	public void keyTyped(KeyEvent e) {
	}

	@Override
	public void keyPressed(KeyEvent e) {
		resetHighlight();
	}

	@Override
	public void keyReleased(KeyEvent e) {
		
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		resetHighlight();
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	private void resetHighlight() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					jPanelSommaire.afficheFonction(getCaretPosition());
					jPanelSommaire.repaint();
				} catch (Exception ble) {
					ble.fillInStackTrace();
				}
			}
		});
	}

}