/***********************************************************************
 * JCubitainer                                                         *
 * Version release date : November 30, 2005                            *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.frame.moteur;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import org.alize.kernel.AKException;
import org.alize.kernel.AKJob;
import org.alize.kernel.AKRuntime;
import org.linotte.frame.Atelier;
import org.linotte.frame.atelier.Inspecteur;
import org.linotte.frame.cahier.Cahier;
import org.linotte.frame.coloration.ProcessStyle;
import org.linotte.frame.latoile.Java6;
import org.linotte.moteur.exception.LectureException;
import org.linotte.moteur.exception.Messages;
import org.linotte.moteur.exception.StopException;
import org.linotte.moteur.exception.SyntaxeException;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Linotte;
import org.linotte.moteur.xml.Version;
import org.linotte.moteur.xml.alize.kernel.ContextHelper;
import org.linotte.moteur.xml.alize.kernel.Job;
import org.linotte.moteur.xml.alize.kernel.RuntimeContext;
import org.linotte.moteur.xml.alize.parseur.ParserContext;
import org.linotte.moteur.xml.alize.parseur.ParserContext.MODE;
import org.linotte.moteur.xml.alize.parseur.Parseur;
import org.linotte.moteur.xml.api.IPrelecture;
import org.linotte.moteur.xml.appels.Appel;
import org.linotte.moteur.xml.appels.Fonction;

import console.Jinotte;

public class FrameProcess extends Thread {

	public static final Color RUN = new Color(107, 169, 128);

	static Linotte lecteur = null;

	private StringBuilder buffer = null;

	public static Atelier applet = null;

	boolean start = false;

	private static FrameProcess this_ = null;

	private boolean test;

	ProcessStyle processStyle = null;

	private static AKRuntime runtime;

	public static long t1, t2 = 0;

	IPrelecture prelecture = new IPrelecture() {

		public void debut() {
			applet.chargement.setVisible(true);
		}

		public void fin() {
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
			}
			applet.chargement.setVisible(false);
		}

	};;

	public FrameProcess(Linotte param, Atelier ap) {
		super("FrameProcess");
		lecteur = param;
		applet = ap;
		// setPriority(MAX_PRIORITY);
	}

	public static void go(StringBuilder f, Linotte param, Atelier ap, boolean test) {

		this_ = new FrameProcess(param, ap);
		this_.test = test;
		this_.buffer = f;
		if (!this_.start) {
			this_.start = true;
			this_.start();
		}
	}

	public static void stopProcess() throws StopException {
		// Arrêt des verbes Attendre
		// TODO, à remonter dans le kernel
		this_.interrupt();
		synchronized (runtime.getJob()) {
			try {
				runtime.getJob().notifyAll();
			} catch (Exception e) {
				e.printStackTrace();
			}
			runtime.getJob().stop();
		}
		// runtime.stopAll();
	}

	public void action() throws InterruptedException {
		applet.getJButtonLire().setEnabled(false);
		applet.getJButtonLire().setVisible(true);
		applet.getJButtonStop().setEnabled(true);
		applet.getJButtonPause().setVisible(false);
		applet.getJButtonTester().setEnabled(false);
		Inspecteur.lock(false);
		// On vide le cache des touches :
		applet.getToile().getPanelLaToile().videTouches();
		Cahier cahierCourant = applet.getCahierCourant();
		// Linotte 2.0, pour l'utilisation des librairies
		Ressources.getInstance().viderUrlLoader();
		try {
			t2 = 0;
			t1 = System.currentTimeMillis();
			Parseur moteurXML = new Parseur();
			ParserContext atelierOutils = new ParserContext(MODE.GENERATION_RUNTIME);
			atelierOutils.prelecture = prelecture;
			atelierOutils.linotte = lecteur;
			runtime = moteurXML.parseLivre(buffer, atelierOutils);
			ContextHelper.populate(runtime.getContext(), lecteur, cahierCourant.getFichier(), null);
			cahierCourant.setCouleur(RUN, false);
			((RuntimeContext) runtime.getContext()).doTest = test;
			runtime.execute();
			cahierCourant.setCouleur(null, true);
		} catch (LectureException e) {

			// Afin d'affiner l'aide utilisateur, on recupère le numéro de ligne
			// :
			List<Integer> numerolignes = new ArrayList<Integer>();
			try {
				lire(buffer, numerolignes);
			} catch (IOException ei) {
				// Tant pis pour le numéro de ligne..
			}

			cahierCourant.setCouleur(null, true);
			if (Version.isBeta())
				e.getException().printStackTrace();
			String erreur = "";
			if (e.getLivre() != null) {
				erreur = "Livre : " + e.getLivre() + "\n";
			}
			boolean nePasAffinerMessage = true;
			// Affichage de la phrase analysée :
			if ((e.getException() instanceof SyntaxeException) && ((SyntaxeException) e.getException()).getPhrase() != null) {
				applet.ecrireErreurTableau("Début de phrase analysée : " + ((SyntaxeException) e.getException()).getPhrase());
				nePasAffinerMessage = false;
			}

			// Affiche de la pile :
			if (e.getKernelStack() != null) {
				Iterator<Appel> i = e.getKernelStack().iterator();
				while (i.hasNext()) {
					Appel a = i.next();
					if (a instanceof Fonction) {
//						Fonction f = (Fonction) a;
						//applet.ecrireErreurTableau(f.toString());			
					}
				}
			}

			int nbligne = Jinotte.retourneLaLigne(numerolignes, e.getPosition());
			if (nbligne > 0)
				erreur += "Ligne " + nbligne + ", ";

			erreur += Messages.retourneErreur(String.valueOf(e.getErreur()));
			if (e.getException().getToken() != null)
				erreur += " : " + e.getException().getToken();
			applet.ecrireErreurTableau(erreur);

			if (e.getLivre() == null) {
				try {
					String cleanDoc = cahierCourant.retourneTexteCahier();

					if (cleanDoc.length() != 0) {

						int position = e.getPosition();
						if (e.getPosition() == cleanDoc.length()) // Bug quand
																	// on est à
																	// la fin du
																	// fichier
							while (cleanDoc.charAt(position) == '\n') {
								position--;
								if (position == -1) {
									position = 0;
									break;
								}
							}

						int debut = position;
						if (nePasAffinerMessage) {
							debut = cleanDoc.lastIndexOf("\n", debut);
							if (debut == -1)
								debut = 0;
						}
						int fin = position;
						fin = cleanDoc.indexOf("\n", fin);
						if (fin == -1)
							fin = cleanDoc.length();

						cahierCourant.setSelectionCachier(debut, fin);
					}
				} catch (Exception e2) {
					e2.printStackTrace();
				}
			}
		} catch (StopException e) {
			cahierCourant.setCouleur(null, true);
		} catch (AKException e) {
			cahierCourant.setCouleur(null, true);
			applet.ecrireErreurTableau(e.getMessage());
			applet.ecrirelnTableau("");
		} catch (StackOverflowError e) {
			cahierCourant.setCouleur(null, true);
			applet.ecrireErreurTableau("Trop de boucles imbriquées ! Le livre doit s'arrêter...");
		} catch (Throwable e) {
			cahierCourant.setCouleur(null, true);
			applet.ecrireErreurTableau("Bravo ! vous avez trouvé un bogue dans l'interprète...");
			applet.ecrireErreurTableau("Merci de m'envoyer un mail avec votre livre");
			applet.ecrireErreurTableau("afin que je puisse le corriger. Merci !");
			applet.ecrireErreurTableau("mail : ronan.mounes@amstrad.eu");
			e.printStackTrace();
			
			int r = JOptionPane.showConfirmDialog(null, "Bravo ! Vous avez trouvé un bogue dans l'interprète Linotte... ça arrive !\n"
					+ "Afin de m'aider à améliorer l'interprète, voulez-vous m'envoyer un courriel contenant l'erreur ?\n"
					+ "(... et n'hésitez pas à attacher au mail votre livre pour m'aider à corriger !)", "Bogue dans l'Atelier Linotte !", JOptionPane.ERROR_MESSAGE, JOptionPane.YES_NO_OPTION,
					Ressources.getImageIcon("face-devilish.png"));
			if (r == 0) {
				try {
					StringWriter sw = new StringWriter();
					PrintWriter pw = new PrintWriter(sw);
					e.printStackTrace(pw);
					String url = "mailto:ronan.mounes@amstrad.eu?subject=" + encode("[Linotte] Bogue : " + e.getMessage());
					url += "&body=" + encode(sw.toString());
					Java6.getDesktop().mail(new URI(url));
				} catch (IOException e1) {
					e1.printStackTrace();
				} catch (URISyntaxException e1) {
					e1.printStackTrace();
				}
			}
		

		} finally {
			cahierCourant.setCouleur(null, false);
			cahierCourant.getEditorPaneCahier().getLineHighlighter().setPositionDeboguage(-1);
			long t3 = System.currentTimeMillis();
			t2 = t3 - t1;
			t1 = 0;
			applet.getJButtonLire().setEnabled(true);
			applet.getJButtonLire().setVisible(true);
			applet.getJButtonStop().setEnabled(false);
			applet.getJButtonPause().setVisible(false);
			applet.getJButtonContinuer().setVisible(false);
			applet.getJButtonTester().setEnabled(true);
			Inspecteur.lock(true);
			prelecture.fin();
			runtime = null;
			this_ = null;
		}
	}

	private String encode(String url) throws UnsupportedEncodingException {
		String encodedurl = URLEncoder.encode(url, "UTF-8");
		return encodedurl;
	}

	@Override
	public void run() {
		if (applet.isVisible()) {
			try {
				action();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public static void continuerDebogage() {
		synchronized (runtime.getJob()) {
			reveillerJob(runtime, false);
		}
	}

	public static void stopDebogage() {
		synchronized (runtime.getJob()) {
			reveillerJob(runtime, true);
		}
	}

	/**
	 * Réveille les jobs en attente lors d'un mode debug
	 * 
	 * @param runtime
	 * @param stop
	 * 
	 */
	private static void reveillerJob(AKRuntime r, boolean stop) {
		synchronized (r.getJob()) {
			if (stop) {
				((Job) r.getJob()).setDebogueur(false);
			}
			r.getJob().notifyAll();
		}
		// Bogue remonté par Pat sur le débogage des fonctions :
		List<AKJob> fils = r.getJob().getSons();
		if (fils != null) {
			synchronized (fils) {
				for (AKJob akJob : fils) {
					synchronized (akJob) {
						if (stop) {
							((Job) akJob).setDebogueur(false);
						}
						akJob.notifyAll();
					}
				}
			}
		}
	}

	private static void lire(StringBuilder buffer, List<Integer> numerolignes) throws IOException {
		// Probablement pas optimum au niveau gestion de la mémoire
		BufferedReader br = new BufferedReader(new StringReader(buffer.toString()));
		int position = 0;
		String str;
		while ((str = br.readLine()) != null) {
			position += str.length() + 1;
			numerolignes.add(position);
		}
		br.close();
	}

}
